@@ -51,7 +51,7 @@ int topcstate = 0;
 
 int dump = 0;
 
-#define IRQCOUNT 100
+#define IRQCOUNT 150
 
 struct irqdata {
 	int active;
@@ -162,16 +162,35 @@ static void do_proc_irq(void)
 		char *c;
 		int nr = -1;
 		uint64_t count = 0;
+		int special = 0;
 		memset(line, 0, sizeof(line));
 		if (fgets(line, 1024, file) == NULL)
 			break;
 		c = strchr(line, ':');
 		if (!c)
 			continue;
-		/* deal with NMI and the like */
-		if (line[0] != ' ' && (line[0] < '0' || line[0] > '9'))
+		/* deal with NMI and the like.. make up fake nrs */
+		if (line[0] != ' ' && (line[0] < '0' || line[0] > '9')) {	
+			if (strncmp(line,"NMI:", 4)==0)
+				nr=20000;
+			if (strncmp(line,"RES:", 4)==0)
+				nr=20001;
+			if (strncmp(line,"CAL:", 4)==0)
+				nr=20002;
+			if (strncmp(line,"TLB:", 4)==0)
+				nr=20003;
+			if (strncmp(line,"TRM:", 4)==0)
+				nr=20004;
+			if (strncmp(line,"THR:", 4)==0)
+				nr=20005;
+			if (strncmp(line,"SPU:", 4)==0)
+				nr=20006;
+			special = 1;
+		} else
+			nr = strtoull(line, NULL, 10);
+
+		if (nr==-1)
 			continue;
-		nr = strtoull(line, NULL, 10);
 		*c = 0;
 		c++;
 		while (c && strlen(c)) {
@@ -186,20 +205,26 @@ static void do_proc_irq(void)
 			continue;
 		while (c && *c == ' ')
 			c++;
-		c = strchr(c, ' ');
-		if (!c) 
-			continue;
-		while (c && *c == ' ')
-			c++;
+		if (!special) {
+			c = strchr(c, ' ');
+			if (!c) 
+				continue;
+			while (c && *c == ' ')
+				c++;
+		}
 		name = c;
 		delta = update_irq(nr, count, name);
 		c = strchr(name, '\n');
 		if (c)
 			*c = 0;
-		if (strcmp(name, "i8042"))
-			sprintf(line2, "    <interrupt> : %s", name);
+		if (strcmp(name, "i8042")) { 
+			if (special) 
+				sprintf(line2, _("   <kernel IPI> : %s"), name);
+			else
+				sprintf(line2, _("    <interrupt> : %s"), name);
+		}
 		else
-			sprintf(line2, "    <interrupt> : %s", _("PS/2 keyboard/mouse/touchpad"));
+			sprintf(line2, _("    <interrupt> : %s"), _("PS/2 keyboard/mouse/touchpad"));
 
 		if (nr > 0 && delta > 0)
 			push_line(line2, delta);
@@ -444,10 +469,11 @@ int main(int argc, char **argv)
 	do_proc_irq();
 	do_proc_irq();
 	do_cpufreq_stats();
+	count_usb_urbs();
 
 	memset(cur_usage, 0, sizeof(cur_usage));
 	memset(cur_duration, 0, sizeof(cur_duration));
-	printf("PowerTOP 1.8    (C) 2007 Intel Corporation \n\n");
+	printf("PowerTOP 1.9    (C) 2007 Intel Corporation \n\n");
 	if (geteuid() != 0)
 		printf(_("PowerTOP needs to be run as root to collect enough information\n"));
 	printf(_("Collecting data for %i seconds \n"), (int)ticktime);
@@ -577,11 +603,11 @@ int main(int argc, char **argv)
 				c++;
 			func = c;
 			if (strcmp(process, "insmod") == 0)
-				process = "<kernel module>";
+				process = _("<kernel module>");
 			if (strcmp(process, "modprobe") == 0)
-				process = "<kernel module>";
+				process = _("<kernel module>");
 			if (strcmp(process, "swapper") == 0)
-				process = "<kernel core>";
+				process = _("<kernel core>");
 			c = strchr(c, '\n');
 			if (strncmp(func, "tick_nohz_", 10) == 0)
 				continue;
@@ -613,7 +639,7 @@ int main(int argc, char **argv)
 					totalevents += interrupt_0 - d;
 			}
 			if (d>0 && d < interrupt_0)
-				push_line("    <interrupt> : extra timer interrupt", interrupt_0 - d);
+				push_line(_("    <interrupt> : extra timer interrupt"), interrupt_0 - d);
 		}
 
 	
@@ -621,6 +647,7 @@ int main(int argc, char **argv)
 			wakeups_per_second = totalevents * 1.0 / ticktime / sysconf(_SC_NPROCESSORS_ONLN);
 			show_wakeups(wakeups_per_second, ticktime, c0 * 100.0 / (sysconf(_SC_NPROCESSORS_ONLN) * ticktime * 1000 * FREQ) );
 		}
+		count_usb_urbs();
 		print_battery();
 		count_lines();
 		sort_lines();
@@ -658,8 +685,6 @@ int main(int argc, char **argv)
 		if (wakeups_per_second < 0)
 			ticktime = 2;
 
-		if (dump)
-			exit(EXIT_SUCCESS);
 		reset_suggestions();
 
 		suggest_kernel_config("CONFIG_USB_SUSPEND", 1,
@@ -700,7 +725,10 @@ int main(int argc, char **argv)
 				      "Beagle is the program that indexes for easy desktop search, however it's \n"
 				      "not very efficient and costs a significant amount of battery life."), 30);
 
-		/* suggest to stop gnome-power-manager only if it shows up in the top 10 and wakes up more than 10 times in the measurement */
+		/* suggest to stop gnome-power-manager *only* if it shows up in the top 10 and wakes up more than 10 times in the measurement */
+		/* note to distribution makers: There is no need to patch this out! */
+		/* If you ship a recent enough g-p-m, the warning will not be there, */
+		/* and if you ship a really old one the warning is really justified. */
 		suggest_process_death("gnome-power-man : schedule_timeout (process_timeout)", "gnome-power-manager", lines, min(linehead,10), 10.0,
 				    _("Suggestion: Disable or remove 'gnome-power-manager' from your system. \n"
 				      "Older versions of gnome-power-manager wake up far more often than \n"
@@ -742,6 +770,11 @@ int main(int argc, char **argv)
 		suggest_writeback_time();
 		suggest_usb_autosuspend();
 
+		if (dump) {
+			print_all_suggestions();
+			exit(EXIT_SUCCESS);
+		}
+
 		if (!key)
 			pick_suggestion();
 		show_title_bar();
